#include "dropshell.hpp"
#include "server_service.hpp"
#include "autocomplete.hpp"
#include "init_user_directory.hpp"
#include "config.hpp"

#include <iostream>
#include <string>
#include <vector>

namespace dropshell {

void print_help() {
    std::cout << "Usage: dropshell [OPTIONS] COMMAND [ARGS]" << std::endl;
    std::cout << std::endl;
    std::cout << "A tool for managing server configurations" << std::endl;
    std::cout << std::endl;
    std::cout << "Commands:" << std::endl;
    std::cout << "  help           Show this help message" << std::endl;
    std::cout << "  version        Show version information" << std::endl;
    std::cout << "  init DIR       Set the local directory for all server configs and backups" << std::endl;
    std::cout << std::endl;
    std::cout << "  servers        Summary of all configured servers" << std::endl;
    std::cout << "  servers NAME   Show details for specific server" << std::endl;
    std::cout << "  templates      List all available templates" << std::endl;
    std::cout << std::endl;
    std::cout << "  install SERVER [SERVICE]       Install/Update service(s)." << std::endl;
    std::cout << "  backup SERVER [SERVICE]        Backup service(s)." << std::endl;
    std::cout << "  COMMAND SERVER [SERVICE]       Run a custom command on service(s)." << std::endl;
    std::cout << std::endl;
    std::cout << "Examples:" << std::endl;
    std::cout << "  dropshell servers" << std::endl;
    std::cout << "  dropshell servers myserver" << std::endl;
    std::cout << "  dropshell init /path/to/directory" << std::endl;
    std::cout << "  dropshell templates" << std::endl;
    std::cout << "  dropshell install myserver myservice" << std::endl;
    std::cout << "  dropshell run myserver myservice status" << std::endl;
    std::cout << "  dropshell backup myserver myservice" << std::endl;
}

} // namespace dropshell


bool parseargs(std::string arg2, std::string arg3, std::string & server_name, std::vector<std::string>& servicelist)
{
    if (arg2.empty()) return false;
    server_name = arg2;

    if (arg3.empty()) {
        servicelist = dropshell::get_server_services(server_name);
    } else {
        servicelist.push_back(arg3);
    }

    return true;
}

std::string safearg(int argc, char *argv[], int index)
{
    if (index >= argc) return "";
    return argv[index];
}

int main(int argc, char* argv[]) {
    try {
        // Load configuration
        if (!dropshell::load_config()) {
            std::cerr << "Error: Failed to load configuration" << std::endl;
            return 1;
        }

        // No arguments provided
        if (argc < 2) {
            dropshell::print_help();
            return 1;
        }

        // Handle commands
        std::string cmd = argv[1];

        auto commands = dropshell::autocomplete_list_commands();
        
        if (cmd == "help" || cmd == "-h" || cmd == "--help") {
            dropshell::print_help();
            return 0;
        }

        if (cmd == "version" || cmd == "-V" || cmd == "--version") {
            dropshell::print_version();
            return 0;
        }

        if (cmd == "servers") {
            if (argc > 2) {
                // Show details for specific server
                dropshell::show_server_details(argv[2]);
            } else {
                // List all servers
                dropshell::list_servers();
            }
            return 0;
        }

        if (cmd == "templates") {
            dropshell::list_templates();
            return 0;
        }

        if (cmd == "init") {
            if (argc < 3) {
                std::cerr << "Error: init command requires a directory argument" << std::endl;
                return 1;
            }
            try {
                dropshell::init_user_directory(argv[2]);
                return 0;
            } catch (const std::exception& e) {
                std::cerr << "Error: " << e.what() << std::endl;
                return 1;
            }
        }

        if (cmd == "autocomplete_list_servers") {
            auto servers = dropshell::autocomplete_list_servers();
            for (const auto& server : servers) {
                std::cout << server << std::endl;
            }
            return 0;
        }

        if (cmd == "autocomplete_list_services") {
            if (argc < 3) {
                std::cerr << "Error: autocomplete_list_services requires a server name" << std::endl;
                return 1;
            }
            auto services = dropshell::autocomplete_list_services(argv[2]);
            for (const auto& service : services) {
                std::cout << service << std::endl;
            }
            return 0;
        }

        if (cmd == "autocomplete_list_commands") {
            for (const auto& command : commands) {
                std::cout << command << std::endl;
            }
            return 0;
        }

        if (cmd == "install") {
            std::string server_name;
            std::vector<std::string> servicelist;
            if (!parseargs(safearg(argc, argv, 2), safearg(argc, argv, 3), server_name, servicelist)) {
                std::cerr << "Error: install command requires server name and optionally service name" << std::endl;
                return 1;
            }
            for (const auto& service_name : servicelist) {
                dropshell::server_service service;
                if (!service.init(server_name, service_name)) {
                    std::cerr << "Error: Failed to initialize service" << std::endl;
                    return 1;
                }
                
                if (!service.install()) {
                    std::cerr << "Error: Failed to install service" << std::endl;
                    return 1;
                }
            }
            return 0;
        }

        if (cmd == "backup") {
            std::string server_name;
            std::vector<std::string> servicelist;
            if (!parseargs(safearg(argc, argv, 2), safearg(argc, argv, 3), server_name, servicelist)) {
                std::cerr << "Error: backup command requires server name and optionally service name" << std::endl;
                return 1;
            }

            for (const auto& service_name : servicelist) {
                dropshell::server_service service;
                if (!service.init(server_name, service_name)) {
                    std::cerr << "Error: Failed to initialize service" << std::endl;
                    return 1;
                }
                if (!service.backup()) {
                    std::cerr << "Backup failed." << std::endl;
                    return 1;
                }
            }
            return 0;
        }

        for (const auto& command : commands) {
            if (cmd == command) {
                std::string server_name;
                std::vector<std::string> servicelist;
                if (!parseargs(safearg(argc, argv, 2), safearg(argc, argv, 3), server_name, servicelist)) {
                    std::cerr << "Error: " << command << " command requires server name and optionally service name" << std::endl;
                    return 1;
                }

                for (const auto& service_name : servicelist) {
                    dropshell::server_service service;
                    if (!service.init(server_name, service_name)) {
                        std::cerr << "Error: Failed to initialize service" << std::endl;
                        return 1;
                    }
                    if (!service.run_command(command)) {
                        std::cerr << command +" failed." << std::endl;
                        return 1;
                    }
                }
                return 0;
            }
        }

        // Unknown command
        std::cerr << "Error: Unknown command '" << cmd << "'" << std::endl;
        dropshell::print_help();
        return 1;

    } catch (const std::exception& e) {
        std::cerr << "Error: " << e.what() << std::endl;
        return 1;
    }
} 