#include "init_user_directory.hpp"
#include "dropshell.hpp"
#include "server_env.hpp"
#include "server_service.hpp"
#include "tableprint.hpp"
#include <iostream>
#include <fstream>
#include <iomanip>
#include <boost/filesystem.hpp>
#include <boost/algorithm/string.hpp>

namespace fs = boost::filesystem;

namespace dropshell {

std::vector<ServerInfo> get_configured_servers() {
    std::vector<ServerInfo> servers;
    std::string user_dir;

    if (!is_config_loaded()) {
        std::cerr << "Error: Config not loaded" << std::endl;
        return servers;
    }
    if (!get_user_directory(user_dir)) {
        std::cerr << "Error: User directory not set" << std::endl;
        return servers;
    }
    fs::path servers_dir = fs::path(user_dir) / "servers";
    if (!fs::exists(servers_dir)) {
        std::cerr << "Error: Servers directory not found" << std::endl;
        return servers;
    }

    for (const auto& entry : fs::directory_iterator(servers_dir)) {
        if (fs::is_directory(entry)) {
            fs::path env_file = entry.path();
            server_env env(env_file.string());
            if (!env.is_valid()) {
                std::cerr << "Error: Invalid server environment file: " << env_file.string() << std::endl;
                continue;
            }
            servers.push_back({
                entry.path().filename().string(),
                env.get_SSH_HOST(),
                env.get_SSH_USER(),
                env.get_SSH_PORT()
            });
        }
    }

    return servers;
}

void list_servers() {
    auto servers = get_configured_servers();

    tableprint tp;
    tp.add_row({"Name", "Address"});
    for (const auto& server : servers) {
        tp.add_row({server.name, server.ssh_host});
    }
    tp.print();
}

void show_server_details(const std::string& server_name) {
    std::string user_dir;
    if (!get_user_directory(user_dir)) {
        std::cerr << "Error: User directory not set" << std::endl;
        return;
    }

    fs::path server_dir = fs::path(user_dir) / "servers" / server_name;
    if (!fs::exists(server_dir)) {
        std::cerr << "Error: Server '" << server_name << "' not found" << std::endl;
        return;
    }

    //---------------------
    std::cout << "Server Configuration: " << server_name << std::endl;
    std::cout << std::string(40, '-') << std::endl;

    server_env env(server_dir.string());

    for (const auto& [key, value] : env.get_variables()) {
        std::cout << key << ": " << value << std::endl;
    }

    //---------------------
    // Check if server is reachable via SSH
    std::string ssh_address = env.get_SSH_HOST();
    if (!ssh_address.empty()) {
        std::cout << std::endl << "Server Status:" << std::endl;
        std::cout << std::string(40, '-') << std::endl;
        
        // Try to connect to the server
        std::string cmd = "ssh -o ConnectTimeout=5 " + ssh_address + " 'echo connected' 2>/dev/null";
        int result = system(cmd.c_str());
        if (result == 0) {
            std::cout << "Status: Online" << std::endl;
            
            // Get uptime if possible
            cmd = "ssh " + ssh_address + " 'uptime' 2>/dev/null";
            int rval = system(cmd.c_str());
            if (rval != 0) {
                std::cout << "Error: Failed to get uptime" << std::endl;
            } 
        } else {
            std::cout << "Status: Offline" << std::endl;
        }
    }

    //---------------------
    // list services, and run healthcheck on each
    std::cout << std::endl;
    std::cout << "Services: " << server_name << std::endl;
    std::cout << std::string(40, '-') << std::endl;

    std::string green_tick = "\033[32m✓\033[0m";
    std::string red_cross = "\033[31m✗\033[0m";

    std::vector<std::string> services = get_server_services(server_name);
    for (const auto& service : services) {
        bool healthy = false;
        std::vector<int> ports;
        server_service ss;
        if (ss.init(server_name, service)) 
        {
            if (ss.is_healthy())
                healthy=true;
            ports = ss.get_ports();
        }
        std::cout << "  " << (healthy ? green_tick : red_cross) << " " << service << ", ports: ";
        std::cout << "(";
        bool first = true;
        for (const auto& port : ports) {
            if (!first) {
                std::cout << ", ";
            }
            std::cout << port;
            first = false;
        }
        std::cout << ")" << std::endl;
   }   

}

} // namespace dropshell 